@@ -1,5 +1,5 @@
 from . GenomeTrack import GenomeTrack
-from .. utilities import file_to_intervaltree, plot_coverage, InputError, transform
+from .. utilities import file_to_intervaltree, plot_coverage, InputError, transform, change_chrom_names
 import numpy as np
 import pyBigWig
 import tempfile
@@ -15,13 +15,15 @@ class BedGraphTrack(GenomeTrack):
                          '.bedGraph', '.bedGraph.gz', '.bedGraph.bgz',
                          '.bdg', '.bdg.gz', '.bdg.bgz']
     TRACK_TYPE = 'bedgraph'
-    OPTIONS_TXT = GenomeTrack.OPTIONS_TXT + """
+    OPTIONS_TXT = GenomeTrack.OPTIONS_TXT + f"""
 color = green
+# To use a different color for negative values
+#negative_color = red
 # To use transparency, you can use alpha
 # default is 1
 # alpha = 0.5
 # the default for min_value and max_value is 'auto' which means that the scale will go
-# from the minimum value found in the region plotted to the maximum value found.
+# roughly from the minimum value found in the region plotted to the maximum value found.
 min_value = 0
 #max_value = auto
 # to convert missing data (NaNs) into zeros. Otherwise, missing data is not plotted.
@@ -75,8 +77,10 @@ show_data_range = true
 # gives the transformed values, if you prefer to see
 # the original values:
 #y_axis_values = original
-file_type = {}
-    """.format(TRACK_TYPE)
+# If you want to have a grid on the y-axis
+#grid = true
+file_type = {TRACK_TYPE}
+    """
     DEFAULTS_PROPERTIES = {'max_value': None,
                            'min_value': None,
                            'show_data_range': True,
@@ -90,11 +94,13 @@ file_type = {}
                            'rasterize': False,
                            'number_of_bins': 700,
                            'type': 'fill',
+                           'region': None,  # Cannot be set manually but is set by tracksClass
                            'transform': 'no',
                            'log_pseudocount': 0,
                            'y_axis_values': 'transformed',
                            'second_file': None,
-                           'operation': 'file'}
+                           'operation': 'file',
+                           'grid': False}
     NECESSARY_PROPERTIES = ['file']
     SYNONYMOUS_PROPERTIES = {'max_value': {'auto': None},
                              'min_value': {'auto': None}}
@@ -106,7 +112,7 @@ file_type = {}
                                          'log10'],
                            'y_axis_values': ['original', 'transformed']}
     BOOLEAN_PROPERTIES = ['show_data_range', 'nans_to_zeros',
-                          'use_middle', 'rasterize']
+                          'use_middle', 'rasterize', 'grid']
     STRING_PROPERTIES = ['file', 'file_type', 'overlay_previous',
                          'orientation', 'summary_method',
                          'title', 'color', 'negative_color',
@@ -123,16 +129,16 @@ file_type = {}
 
     def __init__(self, properties_dict):
         super(BedGraphTrack, self).__init__(properties_dict)
+        self.tbx2 = None
         self.load_file()
 
-        self.tbx2 = None
         self.interval_tree2 = None
 
         if 'second_file' in self.properties['operation']:
             if self.properties['second_file'] is None:
-                raise InputError("operation: {} requires to set the parameter"
-                                 " second_file."
-                                 "".format(self.properties['operation']))
+                raise InputError(f"operation: {self.properties['operation']}"
+                                 " requires to set the parameter"
+                                 " second_file.")
             else:
                 if self.properties['second_file'].endswith(".bgz"):
                     # from the tabix file is not possible to know the
@@ -160,7 +166,7 @@ file_type = {}
             self.log.warning("When an operation is computed"
                              " between 2 files"
                              " a summary_method needs to be"
-                             " used. Will use mean.")
+                             " used. Will use mean.\n")
             self.properties['summary_method'] = 'mean'
 
         if self.properties['operation'] != 'file':
@@ -172,7 +178,7 @@ file_type = {}
                                  "'y_axis_values' was set to 'original'. "
                                  "'y_axis_values' can only be set to "
                                  "'original' when 'transform' is used.\n"
-                                 " It will be set as 'transformed'.")
+                                 " It will be set as 'transformed'.\n")
                 self.properties['y_axis_values'] = 'transformed'
 
     def load_file(self):
@@ -184,10 +190,13 @@ file_type = {}
             try:
                 self.tbx = pysam.TabixFile(self.properties['file'])
             except IOError:
-                self.interval_tree, __, __ = file_to_intervaltree(self.properties['file'])
+                self.interval_tree, __, __ = file_to_intervaltree(self.properties['file'],
+                                                                  self.properties['region'])
         # load the file as an interval tree
         else:
-            self.interval_tree, __, __ = file_to_intervaltree(self.properties['file'])
+            self.interval_tree, __, __ = file_to_intervaltree(self.properties['file'],
+                                                              self.properties['region'])
+
         self.num_fields = None
 
     def _get_row_data(self, row, tbx_var='self.tbx'):
@@ -242,11 +251,11 @@ file_type = {}
         if tbx is not None:
             if chrom_region not in tbx.contigs:
                 chrom_region_before = chrom_region
-                chrom_region = self.change_chrom_names(chrom_region)
+                chrom_region = change_chrom_names(chrom_region)
                 if chrom_region not in tbx.contigs:
                     self.log.warning("*Warning*\nNeither "
                                      + chrom_region_before + " nor "
-                                     + chrom_region + " existss as a "
+                                     + chrom_region + " exists as a "
                                      "chromosome name inside the bedgraph "
                                      "file. This will generate an empty "
                                      "track!!\n")
@@ -260,13 +269,14 @@ file_type = {}
             inttree = eval(inttree_var)
             if chrom_region not in list(inttree):
                 chrom_region_before = chrom_region
-                chrom_region = self.change_chrom_names(chrom_region)
+                chrom_region = change_chrom_names(chrom_region)
                 if chrom_region not in list(inttree):
-                    self.log.warning("*Warning*\nNeither "
-                                     + chrom_region_before + " nor "
-                                     + chrom_region + " existss as a "
-                                     "chromosome name inside the bedgraph "
-                                     "file. This will generate an empty "
+                    self.log.warning("*Warning*\nNo interval was found when "
+                                     "overlapping with both "
+                                     f"{chrom_region_before}:{start_region}-{end_region}"
+                                     f" and {chrom_region}:{start_region}-{end_region}"
+                                     " inside the bedgraph file. "
+                                     "This will generate an empty "
                                      "track!!\n")
                     return score_list, pos_list
             chrom_region = self.check_chrom_str_bytes(inttree, chrom_region)
@@ -290,7 +300,15 @@ file_type = {}
         score_list, pos_list = self.get_scores(chrom_region, start_region, end_region)
         if pos_list == []:
             return
-        score_list = [float(x[0]) for x in score_list]
+        try:
+            score_list = [float(x[0]) for x in score_list]
+        except ValueError as ve:
+            if "could not convert string to float: 'NA'" in str(ve):
+                self.log.warning("*Warning*\nNA were found in the bedgraph"
+                                 " will be replaced by nan.\n")
+                score_list = [float(x[0]) if x[0] != 'NA' else float('nan') for x in score_list]
+            else:
+                raise ve
         if self.properties['use_middle']:
             x_values = np.asarray([(t[0] + t[1]) / 2
                                    for i, t in enumerate(pos_list)
@@ -318,10 +336,9 @@ file_type = {}
                 new_score_list = eval('[' + operation + ' for file in score_list]')
                 new_score_list = np.array(new_score_list)
             except Exception as e:
-                raise Exception("The operation in section {} could not be"
-                                " computed: {}".
-                                format(self.properties['section_name'],
-                                       e))
+                raise Exception("The operation in section "
+                                f"{self.properties['section_name']} could not "
+                                f"be computed: {e}")
             else:
                 score_list = new_score_list
 
@@ -342,10 +359,9 @@ file_type = {}
                 new_score_list = eval('[' + operation + ' for file,second_file in zip(score_list, score_list2)]')
                 new_score_list = np.array(new_score_list)
             except Exception as e:
-                raise Exception("The operation in section {} could not be"
-                                " computed: {}".
-                                format(self.properties['section_name'],
-                                       e))
+                raise Exception("The operation in section "
+                                f"{self.properties['section_name']} could not"
+                                f" be computed: {e}")
             else:
                 score_list = new_score_list
 
@@ -358,7 +374,8 @@ file_type = {}
                       self.size,
                       self.properties['color'],
                       self.properties['negative_color'],
-                      self.properties['alpha'])
+                      self.properties['alpha'],
+                      self.properties['grid'])
 
         ymax = self.properties['max_value']
         ymin = self.properties['min_value']
@@ -368,13 +385,13 @@ file_type = {}
         else:
             ymax = transform(np.array([ymax]), self.properties['transform'],
                              self.properties['log_pseudocount'],
-                             'ymax')
+                             'ymax')[0]
         if ymin is None:
             ymin = plot_ymin
         else:
             ymin = transform(np.array([ymin]), self.properties['transform'],
                              self.properties['log_pseudocount'],
-                             'ymin')
+                             'ymin')[0]
 
         if self.properties['orientation'] == 'inverted':
             ax.set_ylim(ymax, ymin)
@@ -394,7 +411,7 @@ file_type = {}
         # the last position of the pos_list
         bw.addHeader([(chrom_region, pos_list[-1][1])])
         # The starts, ends, score are stored
-        bw.addEntries(np.repeat(chrom_region, len(pos_list)),
+        bw.addEntries([chrom_region] * len(pos_list),
                       [p[0] for p in pos_list],
                       ends=[p[1] for p in pos_list],
                       values=score_list)
@@ -436,7 +453,8 @@ file_type = {}
         super(BedGraphTrack, self).plot_y_axis(ax, plot_axis,
                                                self.properties['transform'],
                                                self.properties['log_pseudocount'],
-                                               self.properties['y_axis_values'])
+                                               self.properties['y_axis_values'],
+                                               self.properties['grid'])
 
     def __del__(self):
         if self.tbx is not None:
