@@ -25,8 +25,9 @@ title =
 height = 2
 # if you want to plot the track upside-down:
 # orientation = inverted
-# if you want to plot the track on top of the previous track. Options are 'yes' or 'share-y'. For the 'share-y'
-# option the y axis values is shared between this plot and the overlay plot. Otherwise, each plot use its own scale
+# if you want to plot the track on top of the previous track. Options are 'yes' or 'share-y'.
+# For the 'share-y' option the y axis values is shared between this plot and the overlay plot.
+# Otherwise, each plot use its own scale
 #overlay_previous = yes
 """
     DEFAULTS_PROPERTIES = {}
@@ -54,28 +55,21 @@ height = 2
         for prop in self.DEFAULTS_PROPERTIES:
             if prop not in self.properties:
                 self.properties[prop] = self.DEFAULTS_PROPERTIES[prop]
-        # use synonymous:
-        for prop in self.SYNONYMOUS_PROPERTIES:
-            synonymous = self.SYNONYMOUS_PROPERTIES[prop]
-            if prop in self.properties and \
-               self.properties[prop] in synonymous:
-                self.properties[prop] = synonymous[self.properties[prop]]
         # check if properties are possible:
         for prop in self.POSSIBLE_PROPERTIES:
             possibles = self.POSSIBLE_PROPERTIES[prop]
             if self.properties[prop] not in possibles:
                 default_value = self.DEFAULTS_PROPERTIES[prop]
-                self.log.warning("*WARNING* {0}: '{1}' for section {2}"
-                                 " is not valid. {0} has "
+                self.log.warning(f"*WARNING* {prop}: '{self.properties[prop]}'"
+                                 " for section "
+                                 f"{self.properties['section_name']}"
+                                 f" is not valid. {prop} has "
                                  "been set to "
-                                 "{3}".format(prop,
-                                              self.properties[prop],
-                                              self.properties['section_name'],
-                                              default_value))
+                                 f"{default_value}.\n")
                 self.properties[prop] = default_value
 
     def plot_y_axis(self, ax, plot_axis, transform='no', log_pseudocount=0,
-                    y_axis='tranformed'):
+                    y_axis='tranformed', only_at_ticks=False):
         """
         Plot the scale of the y axis with respect to the plot_axis
         Args:
@@ -84,6 +78,8 @@ height = 2
             transform: what was the transformation of the data
             log_pseudocount:
             y_axis: 'tranformed' or 'original'
+            only_at_ticks: False: only min_max are diplayed
+                           True: only ticks values are displayed
 
         Returns:
 
@@ -97,79 +93,130 @@ height = 2
             if value % 1 == 0:
                 str_value = str(int(value))
             else:
-                str_value = "{:.1f}".format(value)
+                str_value = f"{value:.1f}"
             return str_value
 
-        ymin, ymax = plot_axis.get_ylim()
+        def untransform(value, transform, log_pseudocount):
+            # given a numeric value, transform and log_pseudocount
+            # return the value before the transformation
+            if transform == 'log':
+                return np.exp(value) - log_pseudocount
+            elif transform == 'log2':
+                return np.exp2(value) - log_pseudocount
+            elif transform == 'log10':
+                return np.power(10, value) - log_pseudocount
+            elif transform == 'log1p':
+                return np.expm1(value)
+            elif transform == '-log':
+                return np.exp(- value) - log_pseudocount
 
-        if transform == 'no' or y_axis == 'transformed':
+        ymin, ymax = plot_axis.get_ylim()
+        # If the ticks are closer than epsilon from the top or bottom
+        # The vertical alignment of label is adjusted
+        epsilon = (ymax - ymin) / 100
+        # When the ymax and ymin are plotted (when there is no grid)
+        # The tick is shifted inside of epsilon_pretty
+        # To avoid to have only half of the width of the line plotted
+        epsilon_pretty = epsilon
+
+        if only_at_ticks:
+            # plot something that looks like this:
+            # tick3 ┐
+            #       │
+            # tick2-|
+            #       │
+            # tick1 ┘
+            if ymin < ymax:
+                ticks_values = [t for t in plot_axis.get_yticks() if t <= ymax and t >= ymin]
+            else:
+                ticks_values = [t for t in plot_axis.get_yticks() if t >= ymax and t <= ymin]
+                ticks_values.sort(reverse=True)
+            labels_pos = ticks_values
+            if transform == 'no' or y_axis == 'transformed':
+                ticks_labels = [value_to_str(t) for t in ticks_values]
+            else:
+                # There is a transformation and we want to display original values
+                ticks_labels = [value_to_str(untransform(t, transform, log_pseudocount)) for t in ticks_values]
+        elif transform == 'no' or y_axis == 'transformed':
             # This is a linear scale
             # plot something that looks like this:
             # ymax ┐
             #      │
             #      │
             # ymin ┘
-
-            # the coordinate system used is the ax.transAxes (lower left corner (0,0), upper right corner (1,1)
-            # this way is easier to adjust the positions such that the lines are plotted complete
+            # adjust the positions such that the lines are plotted complete
             # and not only half of the width of the line.
-            x_pos = [0, 0.5, 0.5, 0]
-            y_pos = [0.01, 0.01, 0.99, 0.99]
+            ticks_values = [ymin + epsilon_pretty, ymax - epsilon_pretty]
+            labels_pos = [ymin, ymax]
+            ticks_labels = [value_to_str(v) for v in [ymin, ymax]]
+            if y_axis == 'transformed' and transform != 'no':
+                if transform == 'log1p':
+                    ymid_str = "log(1 + x)"
+                else:
+                    if log_pseudocount == 0:
+                        ymid_str = f"{transform}(x)"
+                    else:
+                        ymid_str = f"{transform}({log_pseudocount} + x)"
+
+                ax.text(0, (ymax + ymin) / 2, ymid_str,
+                        verticalalignment='center',
+                        horizontalalignment='right', wrap=True)
         else:
-            # ymid is the middle between ymin and ymax
-            # if 0 is between ymin and ymax then ymid is 0
+            # There is a transformation and we want to display original values
             if ymin * ymax < 0:
                 ymid = 0
-                ymid_pos = - ymin / (ymax - ymin)
             else:
                 ymid = (ymin + ymax) / 2
-                ymid_pos = 0.5
-
-            if transform == 'log':
-                ymin, ymid, ymax = np.exp([ymin, ymid, ymax]) - log_pseudocount
-            elif transform == 'log2':
-                ymin, ymid, ymax = np.exp2([ymin, ymid, ymax]) - \
-                    log_pseudocount
-            elif transform == 'log10':
-                ymin, ymid, ymax = np.power(10, [ymin, ymid, ymax]) - \
-                    log_pseudocount
-            elif transform == 'log1p':
-                ymin, ymid, ymax = np.expm1([ymin, ymid, ymax])
-            elif transform == '-log':
-                ymin, ymid, ymax = np.exp(- [ymin, ymid, ymax]) - \
-                    log_pseudocount
-            ymid_str = value_to_str(ymid)
             # plot something that looks like this:
             # ymax ┐
             #      │
             # ymid-|
             #      │
             # ymin ┘
-            x_pos = [0, 0.5, 0.5, 0, 0.5, 0.5, 0]
-            y_pos = [0.01, 0.01, ymid_pos, ymid_pos, ymid_pos, 0.99, 0.99]
-        ymax_str = value_to_str(ymax)
-        ymin_str = value_to_str(ymin)
-        ax.plot(x_pos, y_pos, color='black', linewidth=1, transform=ax.transAxes)
-        ax.text(-0.2, -0.01, ymin_str, verticalalignment='bottom', horizontalalignment='right', transform=ax.transAxes)
-        ax.text(-0.2, 1, ymax_str, verticalalignment='top', horizontalalignment='right', transform=ax.transAxes)
-        if transform != 'no':
-            if y_axis == 'original':
-                ax.text(-0.2, ymid_pos, ymid_str, verticalalignment='center',
-                        horizontalalignment='right', transform=ax.transAxes)
-            else:
-                if transform == 'log1p':
-                    ymid_str = "log(1 + x)"
-                else:
-                    if log_pseudocount == 0:
-                        ymid_str = "{}(x)".format(transform)
-                    else:
-                        ymid_str = "{}({} + x)".format(transform,
-                                                       log_pseudocount)
+            ticks_values = [ymin + epsilon_pretty, ymid, ymax - epsilon_pretty]
+            labels_pos = [ymin, ymid, ymax]
+            ticks_labels = [value_to_str(untransform(v, transform, log_pseudocount)) for v in [ymin, ymid, ymax]]
 
-                ax.text(0, 0.5, ymid_str, verticalalignment='center',
-                        horizontalalignment='right', transform=ax.transAxes,
-                        wrap=True)
+        # The lower label should be verticalalignment='bottom'
+        # if it corresponds to ymin
+        i = 0
+        if (ymin < ymax and ticks_values[i] <= ymin + epsilon) \
+           or (ymin > ymax and ticks_values[i] >= ymin + epsilon):
+            v_al = 'bottom'
+            adjusted_value = labels_pos[i] - epsilon
+        else:
+            v_al = 'center'
+            adjusted_value = labels_pos[i]
+        ax.text(-0.2, adjusted_value, ticks_labels[i],
+                verticalalignment=v_al, horizontalalignment='right')
+        x_pos = [0, 0.5]
+        y_pos = [ticks_values[i]] * 2
+        for i in range(1, len(ticks_values) - 1):
+            ax.text(-0.2, labels_pos[i], ticks_labels[i],
+                    verticalalignment='center',
+                    horizontalalignment='right')
+            x_pos += [0.5, 0, 0.5]
+            y_pos += [ticks_values[i]] * 3
 
+        # The upper label should be verticalalignment='top'
+        # if it corresponds to ymax
+        i = len(ticks_values) - 1
+        if (ymin < ymax and ticks_values[i] >= ymax - epsilon) \
+           or (ymin > ymax and ticks_values[i] <= ymax - epsilon):
+            v_al = 'top'
+        else:
+            v_al = 'center'
+        ax.text(-0.2, labels_pos[i], ticks_labels[i],
+                verticalalignment=v_al, horizontalalignment='right')
+        x_pos += [0.5, 0]
+        y_pos += [ticks_values[i]] * 2
+
+        # Finally plot the line:
+        ax.plot(x_pos, y_pos, color='black', linewidth=1)
+
+        # Set the lims:
+        ax.set_ylim(plot_axis.get_ylim())
+        ax.set_xlim(0, 1)
         ax.patch.set_visible(False)
 
     def plot_label(self, label_ax, width_dpi, h_align='left'):
@@ -206,20 +253,18 @@ height = 2
             try:
                 self.size = float(size)
             except ValueError:
-                self.log.warning("Invalid value: 'type = {}' in section: {}\n"
+                self.log.warning("Invalid value: 'type = "
+                                 f"{self.properties['type']}' in section:"
+                                 f" {self.properties['section_name']}\n"
                                  "A number was expected after ':' and found "
-                                 "'{}'. Will use default."
-                                 "".format(self.properties['type'],
-                                           self.properties['section_name'],
-                                           size))
+                                 f"'{size}'. Will use default.\n")
         else:
             self.plot_type = self.properties['type']
 
         if self.plot_type not in ['line', 'points', 'fill']:
-            self.log.warning("Invalid: 'type = {}' in section: {}\n"
-                             "Will use default."
-                             "".format(self.properties['type'],
-                                       self.properties['section_name']))
+            self.log.warning(f"Invalid: 'type = {self.properties['type']}' in"
+                             f" section: {self.properties['section_name']}\n"
+                             "Will use default.\n")
             self.plot_type = default_plot_type
 
     def process_color(self, param, colormap_possible=False,
@@ -248,36 +293,33 @@ height = 2
         elif self.properties[param][0] == '(':
             try:
                 custom_color = eval(self.properties[param])
-            except ValueError:
-                self.log.warning("*WARNING* {}: '{}' for section {}"
-                                 " is not valid. {} has "
+            except (SyntaxError, NameError) as e:
+                self.log.warning(f"*WARNING*: '{param}' for section"
+                                 f" {self.properties[param]}"
+                                 f" raised an error:\n{e}\n"
+                                 f"{self.properties['section_name']} has "
                                  "been set to "
-                                 "{}".format(param,
-                                             self.properties[param],
-                                             self.properties['section_name'],
-                                             default_value))
+                                 f"{default_value}.\n")
                 valid_color = default_value
             else:
                 if mc.is_color_like(custom_color):
                     valid_color = custom_color
                 else:
-                    self.log.warning("*WARNING* {}: '{}' for section {}"
-                                     " is not valid. {} has "
+                    self.log.warning(f"*WARNING*: '{param}' for section"
+                                     f" {self.properties[param]}"
+                                     " is not valid. "
+                                     f"{self.properties['section_name']} has "
                                      "been set to "
-                                     "{}".format(param,
-                                                 self.properties[param],
-                                                 self.properties['section_name'],
-                                                 default_value))
+                                     f"{default_value}.\n")
                     valid_color = default_value
         if not colormap_possible:
             if valid_color is None:
-                self.log.warning("*WARNING* {}: '{}' for section {}"
-                                 " is not valid. It has "
+                self.log.warning(f"*WARNING*: '{param}' for section"
+                                 f" {self.properties[param]}"
+                                 " is not valid. "
+                                 f"{self.properties['section_name']} has "
                                  "been set to "
-                                 "{}".format(param,
-                                             self.properties[param],
-                                             self.properties['section_name'],
-                                             default_value))
+                                 f"{default_value}.\n")
                 valid_color = default_value
             self.properties[param] = valid_color
             return False
@@ -293,23 +335,25 @@ height = 2
                     try:
                         custom_colors = eval(self.properties[param])
                     except (SyntaxError, NameError) as e:
-                        self.log.warning("Warning: section {}, {} was set as {} but "
-                                         "raises an error:\n{}\nIt will be ignored and"
-                                         " default value will be used."
-                                         "".format(self.properties['section_name'],
-                                                   param, self.properties[param],
-                                                   e))
+                        self.log.warning("Warning: section "
+                                         f"{self.properties['section_name']},"
+                                         f" {param} was set as "
+                                         f"{self.properties[param]} but "
+                                         f"raises an error:\n{e}\nIt will be "
+                                         "ignored and default value will be "
+                                         "used.\n")
                     else:
                         try:
                             valid_colormap = mc.LinearSegmentedColormap.from_list(
                                 'custom', custom_colors, N=100)
                         except ValueError as e:
-                            self.log.warning("Warning: section {}, {} was set as {} but "
-                                             "raises an error:\n{}\nIt will be ignored and"
-                                             " default value will be used."
-                                             "".format(self.properties['section_name'],
-                                                       param, self.properties[param],
-                                                       e))
+                            self.log.warning("Warning: section "
+                                             f"{self.properties['section_name']},"
+                                             f" {param} was set as "
+                                             f"{self.properties[param]} but "
+                                             f"raises an error:\n{e}\nIt will "
+                                             f"be ignored and"
+                                             " default value will be used.\n")
                 else:
                     if self.properties[param] in dir(plt.cm):
                         valid_colormap = self.properties[param]
@@ -317,26 +361,22 @@ height = 2
         # valid_color is None or a valid color or the default value
         # valid_colormap is None or a valid colormap
         if valid_color is None and valid_colormap is None:
-            self.log.warning("*WARNING* {}: '{}' for section {}"
+            self.log.warning(f"*WARNING* {param}: '{self.properties[param]}'"
+                             f" for section {self.properties['section_name']}"
                              " is not valid. It has "
                              "been set to "
-                             "{}".format(param,
-                                         self.properties[param],
-                                         self.properties['section_name'],
-                                         default_value))
+                             f"{default_value}.\n")
             self.properties[param] = default_value
             return default_value_is_colormap
         if colormap_only:
             if valid_colormap is None:
                 # valid_color is not None
-                self.log.warning("*WARNING* {}: '{}' for section {}"
-                                 " is not valid: it is a color whereas"
-                                 " a colormap was expected. It has "
+                self.log.warning(f"*WARNING* {param}: "
+                                 f"'{self.properties[param]}' for section"
+                                 f" {self.properties['section_name']}"
+                                 " is not valid. It has "
                                  "been set to "
-                                 "{}".format(param,
-                                             self.properties[param],
-                                             self.properties['section_name'],
-                                             default_value))
+                                 f"{default_value}.\n")
                 valid_colormap = default_value
             self.properties[param] = valid_colormap
             return True
@@ -348,22 +388,6 @@ height = 2
                 self.properties[param] = valid_colormap
                 return True
 
-    @staticmethod
-    def change_chrom_names(chrom):
-        """
-        Changes UCSC chromosome names to ensembl chromosome names
-        and vice versa.
-        """
-        # TODO: mapping from chromosome names like mithocondria is missing
-        if chrom.startswith('chr'):
-            # remove the chr part from chromosome name
-            chrom = chrom[3:]
-        else:
-            # prefix with 'chr' the chromosome name
-            chrom = 'chr' + chrom
-
-        return chrom
-
     @staticmethod
     def check_chrom_str_bytes(iteratable_obj, p_obj):
         # determine type
